@@ -37,6 +37,7 @@ class QQmlApplicationEngine;
 class DownloadThread;
 class DownloadExtractThread;
 class QTranslator;
+class WriteProgressWatchdog;
 #ifndef CLI_ONLY_BUILD
 class NativeFileDialog;
 #endif
@@ -68,6 +69,9 @@ public:
     Q_PROPERTY(WriteState writeState READ writeState NOTIFY writeStateChanged)
     Q_PROPERTY(bool isOsListUnavailable READ isOsListUnavailable NOTIFY osListUnavailableChanged)
 
+    /* Returns true if the extract size is reliably known (false for gz files which can't store sizes >4GB) */
+    Q_INVOKABLE bool isExtractSizeKnown() const { return _extractSizeKnown; }
+
     /* Set URL to download from, and if known download length and uncompressed length */
     Q_INVOKABLE void setSrc(const QUrl &url, quint64 downloadLen = 0, quint64 extrLen = 0, QByteArray expectedHash = "", bool multifilesinzip = false, QString parentcategory = "", QString osname = "", QByteArray initFormat = "", QString releaseDate = "");
 
@@ -325,6 +329,9 @@ public:
     Q_INVOKABLE bool installElevationPolicy();
     Q_INVOKABLE void restartWithElevatedPrivileges();
 
+    /* Check if audio notification (beep) is available on this system */
+    Q_INVOKABLE bool isBeepAvailable();
+
     /* Performance data export - opens native save dialog and writes performance data to file.
        If native dialogs aren't available, emits performanceSaveDialogNeeded for QML fallback. */
     Q_INVOKABLE bool exportPerformanceData();
@@ -359,6 +366,7 @@ signals:
     void preparationStatusUpdate(QVariant msg);
     void osListPrepared();
     void bottleneckStatusChanged(QVariant status, QVariant throughputKBps);
+    void operationWarning(QVariant message);  // Non-fatal warning during operation (e.g., sync fallback)
     void hwFilterChanged();
     void networkInfo(QVariant msg);
     void cacheVerificationStarted();
@@ -383,6 +391,8 @@ protected slots:
     void startProgressPolling();
     void stopProgressPolling();
     void pollNetwork();
+    void restartWrite(QString reason);  // Restart write in sync mode after stall
+    
     void onSuccess();
     void onError(QString msg);
     void onFileSelected(QString filename);
@@ -439,7 +449,7 @@ protected:
     QTimer _osListRefreshTimer;
     SuspendInhibitor *_suspendInhibitor;
     DownloadThread *_thread;
-    bool _verifyEnabled, _multipleFilesInZip, _online;
+    bool _verifyEnabled, _multipleFilesInZip, _online, _extractSizeKnown;
     QSettings _settings;
     QMap<QString,QString> _translations;
     QTranslator *_trans;
@@ -456,6 +466,10 @@ protected:
     // Performance statistics capture
     PerformanceStats *_performanceStats;
     
+    // Progress watchdog - separate component that monitors for stalls
+    WriteProgressWatchdog* _progressWatchdog = nullptr;
+    bool _forceSyncMode = false;  // Force sync I/O on next write (after recovery restart)
+    
     // Debug options (secret menu)
     bool _debugDirectIO;
     bool _debugPeriodicSync;
@@ -467,6 +481,7 @@ protected:
 
     void _parseCompressedFile();
     void _parseXZFile();
+    void _parseGzFile();
     QString _pubKeyFileName();
     QString _privKeyFileName();
     QString _sshKeyDir();
@@ -475,6 +490,8 @@ protected:
     void _applyCloudInitCustomisationFromSettings(const QVariantMap &s);
     void _continueStartWriteAfterCacheVerification(bool cacheIsValid);
     void scheduleOsListRefresh();
+    void _handleMemoryAllocationFailure(const char* what);
+    void _handleSetupException(const char* what);
 };
 
 #endif // IMAGEWRITER_H
