@@ -24,6 +24,7 @@
 #include <QDebug>
 #include <QProcess>
 #include <QFile>
+#include <QStandardPaths>
 #include <QDir>
 #include <QFileInfo>
 #include <QCryptographicHash>
@@ -246,26 +247,97 @@ void applyQuirks() {
     }
 }
 
+namespace {
+    // Sound files in order of preference (Freedesktop sound theme)
+    static const char* const SOUND_FILES[] = {
+        "/usr/share/sounds/freedesktop/stereo/complete.oga",  // Completion notification
+        "/usr/share/sounds/freedesktop/stereo/bell.oga",      // Bell/alert
+        nullptr
+    };
+    
+    // Find the first available sound file
+    static const char* findSoundFile() {
+        for (int i = 0; SOUND_FILES[i] != nullptr; i++) {
+            if (access(SOUND_FILES[i], R_OK) == 0) {
+                return SOUND_FILES[i];
+            }
+        }
+        return nullptr;
+    }
+    
+    static bool commandExists(const char* cmd) {
+        return !QStandardPaths::findExecutable(QString::fromLatin1(cmd)).isEmpty();
+    }
+}
+
+bool isBeepAvailable() {
+    // canberra-gtk-play uses the system sound theme - no file needed
+    if (commandExists("canberra-gtk-play")) {
+        return true;
+    }
+    
+    // Other mechanisms need a sound file
+    const char* soundFile = findSoundFile();
+    if (soundFile) {
+        if (commandExists("pw-play") || commandExists("aplay") || commandExists("pactl")) {
+            return true;
+        }
+    }
+    
+    // PC speaker beep - no dependencies
+    if (commandExists("beep")) {
+        return true;
+    }
+    
+    qDebug() << "No beep mechanism available on this Linux system";
+    return false;
+}
+
 void beep() {
-    // Try multiple Linux beep mechanisms in order of preference
-    
-    // 1. Try pactl (PulseAudio) beep - most common on modern Linux desktop systems
-    if (QProcess::execute("pactl", QStringList() << "upload-sample" << "/usr/share/sounds/alsa/Front_Left.wav" << "beep") == 0) {
-        QProcess::execute("pactl", QStringList() << "play-sample" << "beep");
-        return;
+    // 1. canberra-gtk-play (XDG Sound Theme - best option, uses system theme)
+    if (commandExists("canberra-gtk-play")) {
+        if (QProcess::execute("canberra-gtk-play", {"--id=complete"}) == 0) {
+            return;
+        }
     }
     
-    // 2. Try system bell via echo (works on most terminals)
-    if (QProcess::execute("echo", QStringList() << "-e" << "\\a") == 0) {
-        return;
+    // Find a sound file for the remaining mechanisms
+    const char* soundFile = findSoundFile();
+    
+    if (soundFile) {
+        // 2. pw-play (PipeWire - default on modern distros including Raspberry Pi OS)
+        if (commandExists("pw-play")) {
+            if (QProcess::execute("pw-play", {soundFile}) == 0) {
+                return;
+            }
+        }
+        
+        // 3. aplay (ALSA - widely available, but only supports WAV)
+        if (commandExists("aplay")) {
+            if (QProcess::execute("aplay", {"-q", soundFile}) == 0) {
+                return;
+            }
+        }
+        
+        // 4. pactl (PulseAudio - legacy systems)
+        if (commandExists("pactl")) {
+            if (QProcess::execute("pactl", {"upload-sample", soundFile, "imager-beep"}) == 0) {
+                QProcess::execute("pactl", {"play-sample", "imager-beep"});
+                return;
+            }
+        }
     }
     
-    // 3. Try beep command if available
-    if (QProcess::execute("beep", QStringList()) == 0) {
-        return;
+    // 5. PC speaker beep command
+    if (commandExists("beep")) {
+        if (QProcess::execute("beep", {}) == 0) {
+            return;
+        }
     }
     
-    // 4. Fallback: just log that beep was requested
+    // 6. System bell via echo (rarely works in GUI environments, but worth trying)
+    QProcess::execute("echo", {"-e", "\\a"});
+    
     qDebug() << "Beep requested but no suitable audio mechanism found on this Linux system";
 }
 
@@ -623,6 +695,10 @@ bool launchDetached(const QString& program, const QStringList& arguments) {
         }
         
         // Grandchild - build argv and exec
+        
+        // Clear AppImage environment before running external tools
+        clearAppImageEnvironment();
+        
         QByteArray programBytes = program.toUtf8();
         std::vector<QByteArray> argBytes;
         std::vector<char*> argv;
@@ -807,4 +883,17 @@ const char* findCACertBundle()
     return nullptr;  // Not found, curl will use its compiled-in default
 }
 
+void clearAppImageEnvironment() {
+    // AppImages set LD_LIBRARY_PATH and LD_PRELOAD to use bundled libraries.
+    // External tools need system libraries instead, otherwise they may fail
+    // due to symbol conflicts (e.g., PAM modules failing with "cannot open
+    // session: Module is unknown", or KDE tools failing with Qt version
+    // mismatches like "version `Qt_6.10' not found").
+    //
+    // This is safe because forked children running external tools don't need
+    // our bundled libraries.
+    unsetenv("LD_LIBRARY_PATH");
+    unsetenv("LD_PRELOAD");
+}
+
 } // namespace PlatformQuirks
\ No newline at end of file
